(* Main, Peter Sestoft 1996, 2002-07-31 *)

open Lambda Mosmlcgi
open Msp
infix &&

fun error msg = (print msg; print "\n<br></body></html>")

fun handler (Lambda.Internal s)    = error ("Internal error: " ^ s)
  | handler (Fail s)               = error ("Failure: " ^ s)
  | handler (Parsing.ParseError _) = 
    error ("Parse error: `" ^ Format.expression ^ "'")
  | handler (Lexer.LexicalError (s,_,_)) = error ("Lexical error: " ^ s)
  | handler (Env.Env s)            = error s
  | handler exn                    = raise exn

fun readlam s = Parser.Input Lexer.Token (Lexing.createLexerString s)

val env0 = foldl Env.addenv Env.empty Lambda.binds0

fun capitalize s : wseq =
    case explode s of 
	c :: cr => $ (implode(Char.toUpper c :: cr))
      | []      => Empty

fun run tracer =
    let val (binds, lam) = readlam Format.expression
 	val env          = foldl Env.addenv env0 binds 
	(* Replace env-bound Vars by Glbs, leave free Vars as they are: *)
	val lam = Env.mkglobal Var env lam
	val strategy = getOpt (cgi_field_string "evalorder", "normal order")
	open Reducers
	fun eval env =
	    case strategy of
		"hybrid applicative order" => hyac env
	      | "hybrid normal order"      => hync env
	      | "call-by-name"             => cbnc env
	      | "head spine reduction"     => hsrc env
	      | "normal order"             => norc env
	      | "call-by-value"            => cbvc env
	      | "applicative order"        => aorc env
	      | s => raise Internal ("Unknown evaluation order (" ^ s ^ ")") 
    in 
	printseq (p (capitalize strategy && $ ":"));
	tracer (eval env) lam 
    end

fun main () =
    let val maxstep = 4000
	open Reducers
    in
	print "Content-type: text/html\n\n";
	print "<html><body bgcolor=\"#fbf2e7\">";
	(case getOpt (cgi_field_string "action", "normalize") of
	     "normalize"         => run (show maxstep)
	   | "trace"             => run (trace maxstep)
	   | "trace with labels" => run (tracelabel maxstep)
	   | "singlestep" => run (single (cgi_field_integer("stepno", 0)))
	   | "singlestep (show last)" => 
		 run (single (Int.max(0, cgi_field_integer("stepno", 0))))
	   | "singlestep (show all)" => 
		 run (singleall (Int.max(0, cgi_field_integer("stepno", 0))))
	   | "dot-count"  => run (dot maxstep)
	   | "count"      => run (count maxstep)
	   | "print abbreviations" => Env.printenv env0
	   | _ => raise Internal "Unknown evaluation tracer");
	print "</body></html>"
    end
    handle exn => handler exn

val _ = main ()
